-- SAMPLE CODE for the TSQL 2012 presentation
-- Created by Steve Stedman
--    http://SteveStedman.com
--    twitter:   @SqlEmt

RAISERROR ('Dont run the whole script, just run sections at a time', 20, 1)  WITH LOG


USE [Master];
set statistics io off;

IF EXISTS(SELECT name FROM sys.databases WHERE name = 'tsql2012')
BEGIN
    ALTER DATABASE [tsql2012] SET SINGLE_USER WITH ROLLBACK IMMEDIATE;
	DROP DATABASE [tsql2012];
END;

USE [Master];
CREATE DATABASE [tsql2012];
GO

USE [tsql2012];




-- Table to be used by Over Clause Rows/Range
CREATE TABLE REVENUE
(
[DepartmentID] int,
[Revenue] int,
[Year] int
);

insert into REVENUE
values (1,10030,1998),(2,20000,1998),(3,40000,1998),
       (1,20000,1999),(2,60000,1999),(3,50000,1999),
       (1,40000,2000),(2,40000,2000),(3,60000,2000),
       (1,30000,2001),(2,30000,2001),(3,70000,2001),
       (1,90000,2002),(2,20000,2002),(3,80000,2002),
       (1,10300,2003),(2,1000,2003), (3,90000,2003),
       (1,10000,2004),(2,10000,2004),(3,10000,2004),
       (1,20000,2005),(2,20000,2005),(3,20000,2005),
       (1,40000,2006),(2,30000,2006),(3,30000,2006),
       (1,70000,2007),(2,40000,2007),(3,40000,2007),
       (1,50000,2008),(2,50000,2008),(3,50000,2008),
       (1,20000,2009),(2,60000,2009),(3,60000,2009),
       (1,30000,2010),(2,70000,2010),(3,70000,2010),
       (1,80000,2011),(2,80000,2011),(3,80000,2011),
       (1,10000,2012),(2,90000,2012),(3,90000,2012);






USE [tsql2012];
-- first lets look at the REVENUE table

SELECT * 
  FROM Revenue;

-- first simple sum and avg aggregates
SELECT sum(Revenue) as TotalRevenue, 
       avg(Revenue) as AverageRevenue,
       count(*) as NumRows
  FROM Revenue;


--First OVER Clause pre SQL 2012
-- http://stevestedman.com/?p=1454
SELECT *,
	   avg(Revenue) OVER (PARTITION by DepartmentID) as AverageDeptRevenue,
	   sum(Revenue) OVER (PARTITION by DepartmentID) as TotalDeptRevenue
FROM REVENUE
ORDER BY departmentID, year;







--ROWS PRECEDING
-- http://stevestedman.com/?p=1454
--  look at the sum of revenue over a trailing 3 year period
SELECT Year, DepartmentID, Revenue,
	   sum(Revenue) OVER (PARTITION by DepartmentID 
						  ORDER BY [YEAR] 
						  ROWS BETWEEN 3 PRECEDING AND CURRENT ROW) as CurrentAndPrev3
FROM REVENUE
ORDER BY departmentID, year;



-- ROWS FOLLOWING
-- http://stevestedman.com/?p=1454
SELECT Year, DepartmentID, Revenue,
	   sum(Revenue) OVER (PARTITION by DepartmentID 
						  ORDER BY [YEAR] 
						  ROWS BETWEEN CURRENT ROW AND 3 FOLLOWING) as CurrentAndNext3
FROM REVENUE
ORDER BY departmentID, year;



--ROWS PRECEDING AND FOLLOWING
SELECT Year, DepartmentID, Revenue,
	   sum(Revenue) OVER (PARTITION by DepartmentID 
						  ORDER BY [YEAR] 
						  ROWS BETWEEN 1 PRECEDING AND 1 FOLLOWING) as BeforeAndAfter
FROM REVENUE
ORDER BY departmentID, year;








-- ROWS UNBOUNDED PRECEDING
-- http://stevestedman.com/?p=1485
SELECT Year, DepartmentID, Revenue,
	   min(Revenue) OVER (PARTITION by DepartmentID 
						  ORDER BY [YEAR] 
						  ROWS UNBOUNDED PRECEDING) as MinRevenueToDate
FROM REVENUE
ORDER BY departmentID, year;




-- ROWS UNBOUNDED FOLLOWING
-- http://stevestedman.com/?p=1485
SELECT Year, DepartmentID, Revenue,
	   min(Revenue) OVER (PARTITION by DepartmentID 
						  ORDER BY [YEAR] 
						  ROWS BETWEEN CURRENT ROW AND UNBOUNDED FOLLOWING) as MinRevenueBeyond
FROM REVENUE
ORDER BY departmentID, year;














-- ROWS vs RANGE UNBOUNDED PRECEDING
SELECT Year, DepartmentID, Revenue,
	   sum(Revenue) OVER (PARTITION by DepartmentID 
						  ORDER BY [YEAR] 
						  ROWS UNBOUNDED PRECEDING) as RowsCumulative,
	   sum(Revenue) OVER (PARTITION by DepartmentID 
						  ORDER BY [YEAR] 
						  RANGE UNBOUNDED PRECEDING) as RangeCumulative
FROM REVENUE
WHERE year between 2003 and 2008
ORDER BY departmentID, year;



-- INSERT A DUPLICATE VALUE FOR RANGE UNBOUNDED PRECEEDING
insert into REVENUE
values (1,10000,2005),(2,20000,2005),(3,30000,2005);

-- same query as above
SELECT Year, DepartmentID, Revenue,
	   sum(Revenue) OVER (PARTITION by DepartmentID 
						  ORDER BY [YEAR] 
						  ROWS UNBOUNDED PRECEDING) as RowsCumulative,
	   sum(Revenue) OVER (PARTITION by DepartmentID 
						  ORDER BY [YEAR] 
						  RANGE UNBOUNDED PRECEDING) as RangeCumulative
FROM REVENUE
WHERE year between 2003 and 2008 
ORDER BY departmentID, year;











-------------------------------------------------------------------
-------------------------------------------------------------------
-------------------------------------------------------------------
-------------------------------------------------------------------









-- IIF
-- http://stevestedman.com/?p=1578
-- assume we want to display an indicator to see if we are above
-- or below average.  First we start with the average over departmentID
SELECT Year, DepartmentID, Revenue, 
	   avg(Revenue) OVER (PARTITION by DepartmentID) as AverageRevenue
FROM REVENUE
ORDER BY DepartmentID, year;


-- without IIF using the CASE statement we would get the following
-- http://stevestedman.com/?p=1578
SELECT Year, DepartmentID, Revenue, AverageRevenue,
   CASE WHEN Revenue > AverageRevenue 
        THEN 'Better Than Average'
        ELSE 'Not Better' 
		END as Ranking
FROM (SELECT Year, DepartmentID, Revenue, 
		     avg(Revenue) OVER (PARTITION by DepartmentID) as AverageRevenue
	    FROM REVENUE ) as t
ORDER BY DepartmentID, year;





-- now the same functionality using IIF and simplifying the code
-- http://stevestedman.com/?p=1578
SELECT Year, DepartmentID, Revenue, AverageRevenue,
   iif(Revenue > AverageRevenue, 'Better Than Average', 'Not Better') as Ranking
FROM (SELECT Year, DepartmentID, Revenue, 
		     avg(Revenue) OVER (PARTITION by DepartmentID) as AverageRevenue
	    FROM REVENUE ) as t
ORDER BY DepartmentID, year;






SELECT IIF(3=2,'true',NULL)    -- ERROR ?

SELECT IIF(1=7,NULL,NULL)      -- ERROR !
--Do you think that IIF actually uses the CASE statement?








-------------------------------------------------------------------
-------------------------------------------------------------------
-------------------------------------------------------------------
-------------------------------------------------------------------




-- CHOOSE
-- returns the item at a specific index
-- http://stevestedman.com/?p=1691

declare @corners as int = 6;
-- the old way using case.
SELECT CASE @corners
		WHEN 1 THEN 'point'
		WHEN 2 THEN 'line'
		WHEN 3 THEN 'triangle'
		WHEN 4 THEN 'square'
		WHEN 5 THEN 'pentagon'
		WHEN 6 THEN 'hexagon'
		WHEN 7 THEN 'heptagon'
		WHEN 8 THEN 'octagon'
		else NULL
	   END;

-- now with choose
declare @corners as int = 6;
SELECT choose(@corners, 'point', 'line', 'triangle', 'square', 
                        'pentagon', 'hexagon', 'heptagon', 'octagon')








-- CHOOSE day of week example
DECLARE @day as int=4
SELECT CHOOSE(@day,'Sunday','Monday', 'Tuesday', 
              'Wednesday','Thursday','Friday','Saturday')


-- Pick 5 people at random from a list of 10
GO
DECLARE @myRandonNum INTEGER;
SET @myRandonNum = cast(Rand() * 10 as int) + 1;
Select @myRandonNum as Num, CHOOSE(@myRandonNum, 'Steve', 'Justin', 'Pete', 'Aaron', 'Nick', 'Kim', 'Heather', 'Adrian', 'Dave', 'David') as Name;
GO 5
















-------------------------------------------------------------------
-------------------------------------------------------------------
-------------------------------------------------------------------
-------------------------------------------------------------------


-- OFFSET / FETCH
-- http://stevestedman.com/?p=1597

use AdventureWorks2012;

SELECT *
FROM person.Person
ORDER BY LastName, FirstName, MiddleName;


SET STATISTICS IO ON;

-- the old way to do paging using a CTE
WITH PeoplePagingCTE AS
(
	SELECT ROW_NUMBER() OVER (ORDER BY LastName, FirstName, MiddleName ASC) as RowNum,
	       *
	FROM person.Person
)
SELECT *
  FROM PeoplePagingCTE
 WHERE RowNum BETWEEN 11 and 20
 ORDER BY LastName, FirstName, MiddleName;

-- the new way to do paging with OFFSET and FETCH
SELECT *
  FROM person.Person
 ORDER BY LastName, FirstName, MiddleName
OFFSET 10 ROWS
 FETCH NEXT 10 ROWS ONLY;


-- how does the performance compare between CTE and OFFSET/FETCH	
-- turn on execution plan and run the 2 queries above again.




--   S on ROWS is optional, so OFFSET 10 ROW is valid
SELECT *
  FROM person.Person
 ORDER BY LastName, FirstName, MiddleName
OFFSET 10 ROW
 FETCH NEXT 10 ROW ONLY;



-- the ORDER BY is required, but doesn't have to make sense?
-- for instance, the following orders by newid() - a uniqueidentifier
-- which orders randomly
SELECT *
  FROM person.Person
 ORDER BY newid()
OFFSET 10 ROW
 FETCH NEXT 10 ROW ONLY;


-- dynamic values
SELECT *
  FROM person.Person
 ORDER BY LastName, FirstName, MiddleName
OFFSET 10 ROWS
 FETCH NEXT (SELECT MONTH(GETDATE())) ROWS ONLY;









-------------------------------------------------------------------
-------------------------------------------------------------------
-------------------------------------------------------------------
-------------------------------------------------------------------








-- FORMAT
-- New string function introducted in TSQL 2012

-- old way
DECLARE @Revenue MONEY = 314159.26
SELECT '$' + CONVERT(VARCHAR(32),@Revenue,1);
-- third parameter on CONVERT of 1 specifies to convert money with comma

-- now with format
DECLARE @Revenue MONEY = 314159.26
SELECT FORMAT(@Revenue,'C');


-- other examples
-- format as the date format for the current culture
SELECT FORMAT(getdate(), 'd');  -- Short Date
SELECT FORMAT(getdate(), 'D');  -- Long Date
SELECT FORMAT(getdate(), 'f');  -- Long date with time... without seconds
SELECT FORMAT(getdate(), 'F');  -- Long date with time... with seconds


-- format as hexidecimal
SELECT FORMAT(1234, 'X');

-- format as a sortable date string
SELECT FORMAT(getdate(), 's');

-- custom format values
SELECT FORMAT(getdate(), 'MMMM dd, yyyy (dddd)');




-- using the culture parameter
DECLARE @Revenue MONEY = 314159.26
SELECT FORMAT(@Revenue,'c','en-us') as English;
SELECT FORMAT(@Revenue,'c','fr-fr') as French;
SELECT FORMAT(@Revenue,'c','de-de') as German;
SELECT FORMAT(@Revenue,'c','ja-JP') as Japanese;




-- examples of culture using the default
DECLARE @Revenue MONEY = 314159.26

SET LANGUAGE Italian
SELECT FORMAT(@Revenue,'c'), FORMAT(getdate(), 'd');

SET LANGUAGE Japanese
SELECT FORMAT(@Revenue,'c'), FORMAT(getdate(), 'd');

SET LANGUAGE British
SELECT FORMAT(@Revenue,'c'), FORMAT(getdate(), 'd');

SET LANGUAGE us_english
SELECT FORMAT(@Revenue,'c'), FORMAT(getdate(), 'd');





--Custom format strings

SELECT FORMAT(1457 ,'#####.####');  --- # - digit replacer, only if the digit exist
SELECT FORMAT(1457.45 ,'#####.####');

SELECT FORMAT(1457 ,'0000.0000');  --- 0- digit replacer, if no digit exist, writes 0.
SELECT FORMAT(1457.45 ,'0000.0000');
SELECT FORMAT(524444333 ,'000-000-0000');   --- cell phone (?)

SELECT FORMAT(0.52 ,'##%'); -- % - similar to P
SELECT FORMAT(0.52 ,'P'); 









-------------------------------------------------------------------
-------------------------------------------------------------------
-------------------------------------------------------------------
-------------------------------------------------------------------







-- CONCAT

-- the old way
SELECT 1 + ' two ' + 3.0 + ' four' -- fails without a cast

SELECT cast(1 as varchar(1024)) + ' two ' + cast(3.0 as varchar(1024)) + ' four'

-- CONCAT in TSQL 2012
SELECT CONCAT(1, ' two ', 3.0, ' four');

-- another example
SELECT 'uniqueidentifier = ' + NEWID();  -- fails
SELECT CONCAT('uniqueidentifier = ', NEWID());


-- PRINT CONCAT TRICK
PRINT'Time '+GETDATE();  -- fails
PRINT 'Time ' + CAST(GETDATE() AS VARCHAR(30));
PRINTCONCAT('Time ',GETDATE());










-------------------------------------------------------------------
-------------------------------------------------------------------
-------------------------------------------------------------------
-------------------------------------------------------------------


USE [tsql2012];

-- SEQUENCE

-- using default where does this one start?
--DROP SEQUENCE myTestSequence;
CREATE SEQUENCE myTestSequence;
select NEXT VALUE FOR myTestSequence;



-- using int... where does this one start?
--DROP SEQUENCE myTestSequence2;
CREATE SEQUENCE myTestSequence2 
    AS INT;
select NEXT VALUE FOR myTestSequence2;

-- using smallint
--DROP SEQUENCE myTestSequence3;
CREATE SEQUENCE myTestSequence3 
    AS SMALLINT;
select NEXT VALUE FOR myTestSequence3;



-- using tinyint... where does this one start?
--DROP SEQUENCE myTestSequence4;
CREATE SEQUENCE myTestSequence4
    AS TINYINT;
select NEXT VALUE FOR myTestSequence4;



-- start it at 0
-- DROP SEQUENCE myTestSequence5;
CREATE SEQUENCE myTestSequence5
 START WITH 0;
select NEXT VALUE FOR myTestSequence5;


--DROP SEQUENCE ordersKeySequence;
CREATE SEQUENCE ordersKeySequence
    AS bigint
	START WITH 100
    INCREMENT BY 1 ;


-- Create a table
--DROP TABLE Orders;
CREATE TABLE Orders
    (OrderID int PRIMARY KEY,
     Name varchar(20) NOT NULL,
     Qty int NOT NULL);
GO
-- Insert three records
INSERT Orders (OrderID, Name, Qty)
    VALUES (NEXT VALUE FOR ordersKeySequence, 'Hat', 2) ;
INSERT Orders (OrderID, Name, Qty)
    VALUES (NEXT VALUE FOR ordersKeySequence, 'Shirt', 1) ;
INSERT Orders (OrderID, Name, Qty)
    VALUES (NEXT VALUE FOR ordersKeySequence, 'Shoes', 1) ;
GO

-- View the table
SELECT * FROM Orders ;
GO


-- Another way to do it.
DECLARE @NextID int ;
SET @NextID = NEXT VALUE FOR ordersKeySequence;
INSERT Orders (OrderID, Name, Qty)
    VALUES (@NextID, 'Belt', 2) ;
GO
SELECT * FROM Orders ;
GO


-- the easy way

DROP TABLE Orders;
CREATE TABLE Orders
    (OrderID int PRIMARY KEY DEFAULT NEXT VALUE FOR ordersKeySequence,
     Name varchar(20) NOT NULL,
     Qty int NOT NULL);
GO



-- Insert three records
INSERT Orders (Name, Qty) VALUES ('Hat', 2) ;
INSERT Orders (Name, Qty) VALUES ('Shirt', 1) ;
INSERT Orders (Name, Qty) VALUES ('Shoes', 1) ;

-- View the table
SELECT * FROM Orders ;
GO




-- TRANSACTIONS
-- DROP SEQUENCE myTestSequence6;
CREATE SEQUENCE myTestSequence6
 START WITH 0;
select NEXT VALUE FOR myTestSequence6;

BEGIN TRANSACTION;
select NEXT VALUE FOR myTestSequence6;
COMMIT TRANSACTION;

select NEXT VALUE FOR myTestSequence6;

-- now what happens if we roll it back??
select NEXT VALUE FOR myTestSequence6;

BEGIN TRANSACTION;
select NEXT VALUE FOR myTestSequence6;
ROLLBACK TRANSACTION;

select NEXT VALUE FOR myTestSequence6;







CREATE SEQUENCE withOverSequence
    AS bigint
	START WITH 100
    INCREMENT BY 1 ;


-- SEQUENCEs with the OVER clause
SELECT NEXT VALUE FOR withOverSequence OVER(ORDER BY year),
	   *
FROM REVENUE
ORDER BY departmentID, year;




-- RESETTING A SEQUENCE
SELECT NEXT VALUE FOR withOverSequence ;

ALTER SEQUENCE withOverSequence
RESTART WITH 1 ;

SELECT NEXT VALUE FOR withOverSequence ;








-- SEQUENCE
-- generating letters with a sequence.
--DROP SEQUENCE lettersSequence;
CREATE SEQUENCE lettersSequence
    AS tinyint
	--START WITH 65
	MINVALUE 65
	MAXVALUE 90
	CYCLE
    INCREMENT BY 1 ;

SELECT char(NEXT VALUE FOR lettersSequence);


--DROP SEQUENCE numberSequence;
CREATE SEQUENCE numbersSequence 
    AS INT
	START WITH 0;
select NEXT VALUE FOR numbersSequence;


-- sample a sequence to generate a letter then numbers for student IDs
--DROP TABLE Students;
CREATE TABLE Students
    (UserID varchar(30) PRIMARY KEY DEFAULT char(NEXT VALUE FOR lettersSequence) + 
	                                  CAST(NEXT VALUE FOR numbersSequence as VARCHAR),
     Name varchar(20) NOT NULL);
GO
-- Insert three records
set nocount on;
INSERT Students (Name) VALUES ('Some Name');
GO 3
SELECT * 
  FROM STUDENTS
 ORDER BY UserID;




 -- SEQUENCE cycle
--DROP SEQUENCE CycleSequence;
CREATE SEQUENCE CycleSequence AS BIGINT
	START WITH 4
	INCREMENT BY 1
	MINVALUE 1
	MAXVALUE 5
	CYCLE
	NO CACHE;

SELECT NEXT VALUE for CycleSequence;




-- take a look at what sequences are available in the database
SELECT *
  FROM sys.sequences;



SELECT seq.name AS [Sequence Name],
       seq.object_id AS [Object ID],
	   seq.create_date AS [Creation Date],
	   seq.modify_date AS [Last Modified Date],
	   CAST(seq.precision AS int) AS [Numeric Precision],
	   CAST(seq.scale AS int) AS [Numeric Scale],
	   ISNULL(seq.start_value,N'''') AS [Start Value],
	   ISNULL(seq.increment,N'''') AS [Increment Value],
	   ISNULL(seq.minimum_value,N'''') AS [Min Value],
	   ISNULL(seq.maximum_value,N'''') AS [Max Value],
	   CAST(seq.is_cycling AS bit) AS [Is Cycle Enabled],
	   ISNULL(seq.cache_size,0) AS [Cache Size],
	   ISNULL(seq.current_value,N'''') AS [Current Value]
  FROM sys.sequences AS seq
 ORDER BY seq.modify_date DESC;





-------------------------------------------------------------------
-------------------------------------------------------------------
-------------------------------------------------------------------
-------------------------------------------------------------------












-- sp_describe_first_result_set
USE [tsql2012];
SET STATISTICS IO OFF;

CREATE TABLE MyTable
(
[DepartmentID] int,
[Revenue] int,
[Year] int,
[Name] varchar(1024)
);

insert into MyTable
values (1,10030,1998, 'xyz'), (2,10,1999, '10'), (3,20,2000, '20 Things');

SELECT * FROM MyTable;

exec sp_describe_first_result_set N'SELECT * FROM MyTable';

-- sp_describe_first_result_set  with parameters
exec sp_describe_first_result_set 
	@tsql = N'SELECT * FROM MyTable WHERE Name = @name',
	@params = N'@name varchar(1024)';

--@browse_information_mode
-- analyzed with no additional information
exec sp_describe_first_result_set N'SELECT * FROM MyTable WHERE Name = @name',
	N'@name varchar(1024)',
	0;


-- analyzed FOR BROWSE
exec sp_describe_first_result_set N'SELECT * FROM MyTable WHERE Name = @name',
	N'@name varchar(1024)',
	1;
-- analyzed as if it was preparing a cursor
exec sp_describe_first_result_set N'SELECT * FROM MyTable WHERE Name = @name',
	N'@name varchar(1024)',
	2;














-------------------------------------------------------------------
-------------------------------------------------------------------
-------------------------------------------------------------------
-------------------------------------------------------------------







-- DATEFROMPARTS(year,month,day)
-- the old way
SELECT Date=cast(Convert(datetime,convert(varchar(10),2012)+'-'+convert(varchar(10),6)+'-'+convert(varchar(10),1), 101)as date)

-- new TSQL DATEFROMPARTS
SELECT DateFROMParts ( 2012, 11, 10 ) AS Date;


--TIMEFROMPARTS (hour,minute,seconds,fractions,precision)
SELECT TimeFROMParts(11,15,20,147,3)



--DATETIMEFROMPARTS (year,month,day,hour,minute,seconds,milliseconds)
SELECT  DateTimeFROMParts(2012, 11, 10, 11, 15, 49, 147)




--DATETIME2FROMPARTS (year,month,day,hour,minute,seconds,fractions,precision)
SELECT DateTime2FROMParts(2012, 11, 10, 11, 15, 49, 147, 4);

-- this will throw an error
--   the precision is smaller than the fraction specified
SELECT DateTime2FROMParts(2012, 11, 10, 11, 15, 49, 147, 1);

--SMALLDATETIMEFROMPARTS(year,month,day,hour,minute)
SELECT SmallDateTimeFROMParts(2012, 11, 10, 11, 15);




--DATETIMEOFFSETFROMPARTS ( year, month, day, hour, minute, seconds, fractions, hour_offset, minute_offset, precision )
SELECT DateTimeOffsetFROMParts (2012, 11, 10, 11, 15, 25, 120, 8, 0, 3);



--EOMONTH 
DECLARE @date DATETIME
SET @date = DATEFROMPARTS(2012, 11, 10)
SELECT EOMONTH ( @date ) AS Result;

-- next month end of month
SELECT EOMONTH ( @date, +1) AS Result;

-- last month end of month
SELECT EOMONTH ( @date, -1) AS Result;











-------------------------------------------------------------------
-------------------------------------------------------------------
-------------------------------------------------------------------
-------------------------------------------------------------------

















-- TSQL Conversion Functions
-- PARSE ( string_value AS data_type [ USING culture ] )
SELECT parse('2012-11-10' as date);

-- this is an invalid date
SELECT parse('2012-11-31' as date);

-- with culture
SELECT parse('06-16-2012' as date USING 'en-us');
SELECT parse('06-16-2012' as date USING 'fr-fr');
SELECT parse('16-06-2012' as date USING 'fr-fr');
SELECT PARSE('Tuesday, 31 January 2012, 21:00' AS datetime USING 'en-GB');


-- TRY_PARSE ( string_value AS data_type [ USING culture ] )
-- similar to parse, but null is returned rather than throwing 
--   an error for invalid dates
SELECT try_parse('2012-11-10' as date);

-- this is an invalid date
SELECT try_parse('2012-11-31' as date);

-- now with culture  
SELECT try_parse('11-25-2012' as date USING 'en-us');
SELECT try_parse('25-11-2012' as date USING 'fr-fr');
SELECT try_parse('2012-11-25' as date USING 'fr-fr');
-- something invalid 11th day of 25th month in French
SELECT try_parse('11-25-2012' as date USING 'fr-fr');





-- TRY_CONVERT ( data_type [ ( length ) ], expression [, style ] )

SET DATEFORMAT dmy;
SELECT TRY_CONVERT(datetime2, '12/31/2010') AS Result;


SELECT 
    CASE WHEN TRY_CONVERT(float, 'test') IS NULL 
    THEN 'Cast failed'
    ELSE 'Cast succeeded'
END AS Result;



SELECT *
FROM MyTable;


SELECT *, try_convert(float, Name) as newValue
FROM MyTable;

-- now for the old convert
SELECT *, convert(float, Name) as newValue
FROM MyTable;


-- with style
declare @myDate as datetime;
set @myDate = '12/31/2010';

SELECT TRY_CONVERT(varchar(max), @myDate , 101) AS Result; -- US
SELECT TRY_CONVERT(varchar(max), @myDate , 103) AS Result; -- British/French
SELECT TRY_CONVERT(varchar(max), @myDate , 111) AS Result; -- Japan









-------------------------------------------------------------------
-------------------------------------------------------------------
-------------------------------------------------------------------
-------------------------------------------------------------------









-- THROW
SELECT 1/0

CREATE TABLE ErrorLog(ErrorTime datetime, Severity varchar(max), ErrMsg varchar(max))
GO


-- Prior to SQL 2012

BEGIN TRY
  DECLARE @Number int = 1 / 0;
END TRY
BEGIN CATCH
  -- Log the error info, then re-throw it
  INSERT INTO ErrorLog VALUES(GETDATE(), ERROR_SEVERITY(), ERROR_MESSAGE());
  DECLARE @msg NVARCHAR(MAX) = ERROR_MESSAGE()
  RAISERROR (@msg, 16, 1)
  -- error is 50000, rather than the original error value
END CATCH



-- SQL2012 with THROW
BEGIN TRY
  DECLARE @Number int = 1 / 0;
END TRY
BEGIN CATCH
  -- Log the error info, then re-throw it
  INSERT INTO ErrorLog VALUES(GETDATE(), ERROR_SEVERITY(), ERROR_MESSAGE());
  THROW;
END CATCH


SELECT * FROM ErrorLog;

